/* file_methods.cpp

   C wrapper for Windows MIDI interface, callable by Java programs
*/

#include "StubPreamble.h"
#include "InputFile.h"
#include "OutputFile.h"

static char fileNameBuf[50];
static int fileNameIdx = 0;
static char lineBuf[100];
static int lineBufIdx = 0;

__declspec(dllexport) /*boolean*/ long InputFile_readLine(struct HInputFile* myFile)
{
	FILE* fp = (FILE*)unhand( myFile )->fd;
	char* result = fgets( &lineBuf[0], 100, fp );
	lineBufIdx = 0;
	if ( result == NULL ) {
		return( FALSE );
	} else {
		// wipe out cr/lf at end of line returned by fgets
		while ( strlen( lineBuf) && 
			(( lineBuf[ strlen( lineBuf ) - 1 ] == 0xa ) ||
			 ( lineBuf[ strlen( lineBuf ) - 1 ] == 0xd ))) {
			lineBuf[ strlen( lineBuf ) - 1 ] = 0;
		}
		return( TRUE );
	}
}

__declspec(dllexport) unicode InputFile_readChar(struct HInputFile* myFile)
{
	int result = lineBuf[ lineBufIdx ];
	if ( lineBuf[ lineBufIdx ] != 0 ) {
		lineBufIdx++;
	}
	return( result );
}

__declspec(dllexport) void InputFile_initFileName(struct HInputFile* myFile)
{
	fileNameIdx = 0;
}
__declspec(dllexport) void InputFile_putFileNameChar(struct HInputFile* myFile,long cVal)
{
	char x = cVal;
	if ( fileNameIdx < 49 ) {
		if ( x == '/' ) {
			x = '\\';
		}
		fileNameBuf[fileNameIdx] = x;
		fileNameIdx++;
		fileNameBuf[fileNameIdx] = 0;
	}
}


__declspec( dllexport ) long InputFile_open(struct HInputFile* myFile)
{
	FILE* fp;

	printf( "InputFile opening: '%s'\n", fileNameBuf );

	fp = fopen( fileNameBuf, "rb" );
	unhand( myFile )->fd = (long)fp;
	if ( fp == NULL ) {
		return( FALSE );
	} else {
		return( TRUE );
	}
}


__declspec( dllexport ) void InputFile_close(struct HInputFile* myFile)
{
	printf( "InputFile closing file\n" );
	fclose( (FILE*)unhand( myFile )->fd );
}


__declspec( dllexport ) long InputFile_read( struct HInputFile* myFile, HArrayOfByte* bytes, long n )
{
	long result;
	printf( "reading %d bytes, ptr is %x\n", n, unhand( bytes ));
	result = fread( unhand( bytes ), n, 1, (FILE*)unhand( myFile )->fd );
	return( result );
}

__declspec(dllexport) long InputFile_readByte(struct HInputFile* myFile)
{
	long result = fgetc( (FILE*)unhand( myFile )->fd );
	return( result );
}

#define OutFileNameSize 100
static char outFileName[OutFileNameSize];
static int outFileIdx = 0;

__declspec( dllexport ) long OutputFile_open(struct HOutputFile* myFile)
{
	/*FILE* fp = fopen( (char*)unhand( myFile )->path, "wb" );*/
	FILE* fp = fopen( outFileName, "wb" );
	unhand( myFile )->fd = (long)fp;
	printf( "Opened the OutputFile '%s'\n", outFileName );
	if ( fp == NULL ) {
		return( FALSE );
	} else {
		return( TRUE );
	}
}


__declspec(dllexport) void OutputFile_initOutFileName(struct HOutputFile *x)
{
	outFileIdx = 0;
}

__declspec(dllexport) void OutputFile_putOutFileNameChar(struct HOutputFile *x,unicode cval)
{
	if ( outFileIdx < ( OutFileNameSize - 1 )) {
		if ( cval == '/' ) {
			cval = '\\';
		}
		outFileName[ outFileIdx ] = cval;
		outFileIdx++;
		outFileName[ outFileIdx ] = 0;
	}
}

static char outStr[100];
static int outStrIdx = 0;
__declspec(dllexport) void OutputFile_initOutFileStr(struct HOutputFile *x)
{
	outStrIdx = 0;
}

__declspec(dllexport) void OutputFile_putOutFileStrChar(struct HOutputFile *x,unicode cval)
{
	if ( outStrIdx < 99 ) {
		outStr[ outStrIdx ] = cval;
		outStrIdx++;
		outStr[ outStrIdx ] = 0;
	}
}

__declspec(dllexport) void OutputFile_writeStr(struct HOutputFile *myFile)
{
	fprintf( (FILE*)unhand( myFile )->fd, "%s\n", outStr );
}


__declspec( dllexport ) void OutputFile_close(struct HOutputFile* myFile)
{
	printf( "Closed the OutputFile\n" );
	fclose( (FILE*)unhand( myFile )->fd );
}

__declspec( dllexport ) long OutputFile_write(struct HOutputFile* myFile,HArrayOfByte* bytes, long n)
{
	long result;
    char* scan;
	printf( "Writing to the OutputFile %d bytes, ptr is %x\n", n, unhand( bytes ));
	scan = unhand( bytes );
	printf( "Writing byte %d\n", *scan );
	result = fwrite( unhand( bytes ), n, 1, (FILE*)unhand( myFile )->fd );
	return( result );
}
